%% This code computes population breakdown frontiers in the numerical
% illustration and Monte Carlo simulations of Masten and Poirier (2019).
%
function BF_illustration(dgpChoice)

%% Turn on if you want the shaded BF pictures for the intro of the paper
introFigures = 1;

% Manually change this to obtain axis labels for the slides (and turn
% off axis labels for the paper figures)
axisLabelsOn = 0;

%% Set dgp parameters
params = setDgpParams(dgpChoice);

%% Set grid parameters

% Here I use the same grid parameters as in estimation. This is mainly 
% because for plots and comparisons the number of cgrid points must be the
% same. If you want to get a smoother picture for a population BF, then
% just change Ncgrid in this file
run setGridParams.m;

gridParams.Ngrid = 100;

%% Compute population breakdown frontiers

params.p1 = params.p1true;
params.p0 = params.p0true;
tic
[BFvec,intPieceVec,minPieceVec] = ...
    computeBFvec(cgrid,plowerbarGrid,params,gridParams,z);
toc


trunc.top = 0;
trunc.bottom = 1;
BCvec = computeBCvec(intPieceVec,minPieceVec,plowerbarGrid,trunc);

%% Compute parameter under point identification, (tau,c) = (0,0)

trueProbY1greaterY0 = 1-UBintegralPiece(params,gridParams,0,0)

%% Save output
save(sprintf('../output/BF_MCtruth_dgp%d.mat',dgpChoice));

%% Plot population breakdown frontiers

for j=1:length(plowerbarGrid)
    plowerbar = plowerbarGrid(j);
    BF = BFvec(:,j);

if introFigures == 0

    hold on;
    h=figure('visible','off');
    plot(cgrid,BF,'LineWidth',2,'Color','black');
    set(gca,'fontsize',15);
    title(strcat('Breakdown frontier, $\underline{p}$ = ',...
        sprintf('%2.2g',plowerbar)),'Interpreter','latex');
    xlabel('Deviation from full independence');
    ylabel('Deviation from rank invariance');
    box off;
    axis([0,cupperbar,0,1]);
    print(h,'-deps',...
        sprintf('../images/BFtrue_dgp%d_plowerbar%2.2g.eps',...
            dgpChoice,plowerbar));

    hold off;

elseif introFigures == 1
    
    h = figure('visible','off');
    hAxes = axes; % This is only needed b/c we move the x-axis label down

    % Shade area below curve
    area(cgrid,BF,'FaceColor',[109 182 255]/255);

    % Share area above curve 
    baseval = 1;
    hold on;
    area(cgrid,BF,baseval,'FaceColor',[255 255 109]/255);

    set(gca,'fontsize',40);
    set(gca,'Xtick',[],'Ytick',[]);
    
    if axisLabelsOn == 0
        set(gca,'XTickLabel','','YTickLabel','');
                
        % To place the title below the plot, we put it in the x-axis label
        xlabel(strcat('$\underline{p}$ = ',...
            sprintf('%2.2g',plowerbar)),'Interpreter','latex');   
        
        % Now put some space between the label and the axis
        moveLabel('x',10,h,hAxes);

    elseif axisLabelsOn == 1
        title(strcat('$\underline{p}$ = ',...
            sprintf('%2.2g',plowerbar)),'Interpreter','latex');
    
        ylabel('$t$','Interpreter','LaTex','FontSize',40,'position',[-0.02,1],'rotation',0);
        xlabel('$c$','Interpreter','LaTex','FontSize',40,'position',[0.5,-0.05]);
    end
    
    box off;
    axis([0,cupperbar,0,1]);

    plot(cgrid,BF,'LineWidth',2,'Color','black');
    hold off;
    print(h,'-deps',...
        sprintf('../images/BFtrue_forIntro_dgp%d_plowerbar%2.2g.eps',...
            dgpChoice,plowerbar));
        % Note that the file name has to be changed manually for importing
        % into the tex code, because we can't have decimal points in
        % latex graphics file names.
    
end

end

end