function cdfValue = Flower(params,y,x,c,Data,h,DataBootstrap,epsilon,wEval)
  
    if params.covariatesOn == 0
        
        if x == 1
            px = params.p1;
        elseif x == 0
            px = params.p0;
        end
        
        % 4 = Numerical illustration; use true population functions
        % 6 = Use estimated cdfs and quantile functions
        % 8 = Estimated functions with the bootstrap data
        if nargin == 4
            FYX = trueConditionalCDF_MC(params,y,x);
        elseif nargin == 6
            FYX = smoothedConditionalCDF(y,x,Data,h);
        elseif nargin == 8
            FYX = smoothedConditionalCDF(y,x,Data,h,[],DataBootstrap,epsilon);
        end
        
        cdfValue = Flower_subFun(FYX,px,c);  
        
    elseif params.covariatesOn == 1
        
        if nargin == 4
            WsingleSuppSize = params.WsingleSuppSize;
        else
            WsingleSuppSize = Data.WsingleSuppSize;
        end
        
        if nargin == 9
            
            pX1givenW = params.p1hatCond;
            
            if x == 1
                px = pX1givenW(wEval);
            elseif x == 0
                px = 1-pX1givenW(wEval);
            end

            FYX = smoothedConditionalCDF(y,x,Data,h,wEval,DataBootstrap,epsilon);
            
            % Here we DO NOT average over W since we want to output the
            % cdf bound functions conditional on W
            cdfValue = Flower_subFun(FYX,px,c);
            
        else
        
            cdfValuesCond = nan(WsingleSuppSize,1);

            % Loop over values of the covariate support points.
            for i=1:WsingleSuppSize

                WsuppPointIndex = i;

                % Pick the correct conditional treatment probability
                % See the discussion in Fupper.m
                if nargin == 4
                    pX1givenW = [params.p1w0true params.p1w1true];
                else
                    pX1givenW = params.p1hatCond; % For empirical application
                end

                if x == 1
                    px = pX1givenW(i);
                elseif x == 0
                    px = 1-pX1givenW(i);
                end

                if nargin == 4
                    FYX = trueConditionalCDF_MC(params,y,x,WsuppPointIndex);
                elseif nargin == 6
                    FYX = smoothedConditionalCDF(y,x,Data,h,WsuppPointIndex);
                elseif nargin == 8
                    FYX = smoothedConditionalCDF(y,x,Data,h,WsuppPointIndex,DataBootstrap,epsilon);
                end

                cdfValuesCond(i) = Flower_subFun(FYX,px,c);

            end

            % Law of total probability
            cdfValue = cdfValuesCond' * params.pW';
        
        end
    end

end