% This function is used in Flower.m.
%
function cdfValue = Flower_subFun(FYX,px,c)
    %% Case 1: Small values of c
    if c < min(px,1-px)
 
        cdfValue = max(FYX*px/(px+c), FYX*px/(px-c) - c/(px-c));
        
    %% Case 2A: Intermedeiate values of c
    elseif (px <= c) && (c < 1-px)
        % In this case p(1-x) is above c.
        % We want the bound for x, however
        
        % > 1 can happen for the bootstrap, with epsilon large enough
        % So, for valid cdfs we really only define need to define this case
        % for == 1, we need to extend it to >= 1 to accomodate this large
        % epsilon case
        if FYX >= 1

            cdfValue = 1;

        elseif FYX < 1

            cdfValue = FYX*px/(px+c);

        end
        
    %% Case 2B: Intermedeiate values of c
    elseif (1-px <= c) && (c < px)
        % In this case p(x) is above c.
        % We want the bound for x.

        if FYX <= c/((1-px+c)*px)

            cdfValue = FYX*px;

        elseif FYX > c/((1-px+c)*px)

            cdfValue = FYX*px/(px-c) - c/(px-c);

        end
        
    %% Case 3: Large values of c    
    elseif c >= max(px,1-px)

        % See comment in case 2A
        if FYX >= 1

            cdfValue = 1;

        elseif FYX < 1

            cdfValue = FYX*px;

        end
        
    else
        error('No valid c case.');
    end
    
    % With large enough epsilon, it is possible that Flower will be be
    % outside of [0,1], and that is OK.
    
end