function cdfValue = Fupper(params,y,x,c,Data,h,DataBootstrap,epsilon,wEval)
    
    if params.covariatesOn == 0
        
        if x == 1
            px = params.p1;
        elseif x == 0
            px = params.p0;
        end        
        
        if nargin == 4
            FYX = trueConditionalCDF_MC(params,y,x);
        elseif nargin == 6
            FYX = smoothedConditionalCDF(y,x,Data,h);
        elseif nargin == 8
            FYX = smoothedConditionalCDF(y,x,Data,h,[],DataBootstrap,epsilon);
        end

        cdfValue = Fupper_subFun(FYX,px,c);
        
    elseif params.covariatesOn == 1

        if nargin == 4
            WsingleSuppSize = params.WsingleSuppSize;
        else
            WsingleSuppSize = Data.WsingleSuppSize;
        end
        
        if nargin == 9
            
            pX1givenW = params.p1hatCond;
            
            if x == 1
                px = pX1givenW(wEval);
            elseif x == 0
                px = 1-pX1givenW(wEval);
            end

            FYX = smoothedConditionalCDF(y,x,Data,h,wEval,DataBootstrap,epsilon);
            
            % Here we DO NOT average over W since we want to output the
            % cdf bound functions conditional on W
            cdfValue = Fupper_subFun(FYX,px,c);
            
        else

            cdfValuesCond = nan(WsingleSuppSize,1);

            % Loop over values of the covariate support points.
            for i=1:WsingleSuppSize

                WsuppPointIndex = i;

                % Pick the correct conditional treatment probability:
                % it should be p_{x|w} where 
                % w=value corresponding to WsuppPointIndex
                % x=same as x inputed into the function
                %
                % In setDgpParams.m we define
                %
                %   params.p1hatCond = [params.p1w1true params.p1w0true]'
                %
                % NOTE the index is switched---w=1 comes first THEN w=0
                % So here I define a more natural vector:
                if nargin == 4
                    pX1givenW = [params.p1w0true params.p1w1true];
                else
                    pX1givenW = params.p1hatCond; % For empirical application
                end

                if x == 1
                    px = pX1givenW(i);
                elseif x == 0
                    px = 1-pX1givenW(i);
                end

                if nargin == 4
                    FYX = trueConditionalCDF_MC(params,y,x,WsuppPointIndex);                
                elseif nargin == 6
                    FYX = smoothedConditionalCDF(y,x,Data,h,WsuppPointIndex);
                elseif nargin == 8
                    FYX = smoothedConditionalCDF(y,x,Data,h,WsuppPointIndex,DataBootstrap,epsilon);
                end

                cdfValuesCond(i) = Fupper_subFun(FYX,px,c);

            end

            % Law of total probability
            cdfValue = cdfValuesCond' * params.pW';
        
        end
    end
    
end