function qValue = Qlower(params,tau,x,c,Data,h,DataBootstrap,epsilon)

    if params.covariatesOn == 0

        if x == 1
            px = params.p1;
        elseif x == 0
            px = params.p0;
        end

        tEval = Qlower_get_tEval(px,c,tau);

        if nargin == 4    
            qValue = trueQuantileFunction_MC(params,tEval,x);
        elseif nargin == 6
            qValue = smoothedQuantile(tEval,x,Data,h);
        elseif nargin == 8
            qValue = smoothedQuantile(tEval,x,Data,h,[],DataBootstrap,epsilon);
        end
        
    elseif params.covariatesOn == 1
    
        % When we have covariates, our only choice here is to invert
        % the function Fupper, which has already accounted for
        % covariates in it.
        
        if nargin == 4
            Ymin = params.YsuppLB;
            Ymax = params.YsuppUB;
            
            % Needed for the inversion step below
            h.x0 = 0;
            h.x1 = 0;
            
            Nygrid = 500;

        else
            Ymin = min(Data.Y);
            Ymax = max(Data.Y);
            
            Nygrid = 100;
        
        end
        
        ygrid = linspace(Ymin,Ymax,Nygrid);
        tauvals = nan(1,length(ygrid));

        if nargin == 4

            for i=1:length(ygrid)
                y = ygrid(i);

                tauvals(i) = Fupper(params,y,x,c);

            end
            
        elseif nargin == 6

            for i=1:length(ygrid)
                y = ygrid(i);

                tauvals(i) = Fupper(params,y,x,c,Data,h);

            end

        elseif nargin == 8
            
            if params.storeFbounds == 1
            
                cIndex = find(DataBootstrap.cgrid == c);

                if x == 0
                    xIndex = 1;
                elseif x == 1
                    xIndex = 2;
                end

                tauvals = DataBootstrap.Fupper(:,xIndex,cIndex);

            elseif params.storeFbounds == 0
                for i=1:length(ygrid)
                    y = ygrid(i);

                    tauvals(i) = Fupper(params,y,x,c,Data,h);

                end
            end
        end

        % When the desired quantile doesn't appear in the range of the cdf
        if tau < min(tauvals)
            qValue = Ymin;

        elseif tau > max(tauvals)
            qValue = Ymax;
            
        else
            % Two cases: If we are smoothing, then we can invert the cdf by
            % interpolation. (Since interpolation requires a strictly
            % increasing vector.) Otherwise, we compute the left-inverse.

            if (h.x0 == 0) || (h.x1 == 0)
                
                if nargin == 4
                % Code for dealing with vector tau
                
                    tauvalsRep = repmat(tauvals,length(tau),1);
                    tauRep = repmat(tau',1,length(tauvals));

                    % rows = values of tau
                    % columns = CDFvalues
                    CDFisLarger = (tauvalsRep >= tauRep);

                    CDFindex = length(tauvals) - sum(CDFisLarger,2) + 1;

                    qValue = ygrid(CDFindex);                
                
                else
                % Scalar tau case
                    
                    CDFisLarger = (tauvals >= tau);
                    CDFindex = length(tauvals) - sum(CDFisLarger) + 1;

                    qValue = ygrid(CDFindex);

                end

            else
                % This can be NaN if tau is outside of tauvals. However, in that
                % case the line resets val appropriately.
                qValue = interp1(tauvals, ygrid, tau);            
            end            
        end

        if isnan(qValue)
            error('Quantile inversion in Qlower.m not working correctly.');
        end

    end

    if isnan(qValue)
        display('Qlower is NaN.');
    end
end