% Combo grid search and Newton's method
function value = UBminpiece(params,z,c,Data,h,DataBootstrap,epsilon)
    opts = optimset('Diagnostics','off', 'Display','off');
    
    Nagrid = 25;
    
    if nargin == 3
        % This grid is used when computing the true BF in the MC dgp.
        % Note that it has to automatically adjust based on what the true
        % support of Y1 and Y0 is in the dgp
        
        Y0suppLB = trueQuantileFunction_MC(params,0,0);
        Y0suppUB = trueQuantileFunction_MC(params,1,0);
        
        Y1suppLB = trueQuantileFunction_MC(params,0,1);
        Y1suppUB = trueQuantileFunction_MC(params,1,1);
        
        YsuppLB = min(Y0suppLB,Y1suppLB);
        YsuppUB = max(Y0suppUB,Y1suppUB);
        
        agrid = linspace(YsuppLB,YsuppUB,Nagrid);
        
    elseif nargin >= 4
        agrid = ...
            linspace(min(Data.Y)+z,max(Data.Y)+z,Nagrid);    
        % The objective function is Flower1(a) - Fupper0(a-z), so a rough
        % approximation of its support is the range of the data, expanded 
        % by z
    end
    
    %% 1. Pick starting value via grid search
    fval = nan(length(agrid),1);
    for i=1:length(agrid)
        a = agrid(i);
        
        if nargin == 3
            fval(i) = min(UBobjfun(params,a,z,c),0);
        elseif nargin == 5
            fval(i) = min(UBobjfun(params,a,z,c,Data,h),0);
        elseif nargin == 7
            fval(i) = ...
                min(UBobjfun(params,a,z,c,Data,h,DataBootstrap,epsilon),0);
        end
        
    end
    
    [valueGridSearch, index] = min(fval);
    
    a0 = agrid(index);
    
    %% 2. Minimize function via Newton's method
    if nargin == 3
        [~,fval] = fminunc(@(a) UBobjfun(params,a,z,c),a0,opts);        
    elseif nargin == 5
        [~,fval] = fminunc(@(a) UBobjfun(params,a,z,c,Data,h),a0,opts); 
    elseif nargin == 7
        [~,fval] = fminunc(@(a) ...
            UBobjfun(params,a,z,c,Data,h,DataBootstrap,epsilon),a0,opts);
    end
    
    value = min(fval,0);  
    
    if value > valueGridSearch
        display('Error in UBminpiece: Newtons method does worse than grid search.');
    end

end