function compileAllEmpiricalResults()
clear all;
clc;

% POSSIBLE ISSUE: See "alphaVal = 0.05" below.
% I had used alpha as the variable name, but in the newest version of 
% Matlab alpha is a function so there is a conflict, which caused this line
% below to break. If there are issues, check BF_empirical_main_v2.m and 
% replace "alpha" with "alphaVal" everywhere.

epsilonGridEmpirical = computeEmpiricalEpislonGrid();

%% Load data
params.covariatesOn = 1;
Nbootstrap = 1000;
Ncgrid = 25;
gridParams.Ngrid = 100;
plowerbarGrid = [0.1 0.25 0.5 0.75 0.9];

Empirical.BFvec = nan(Ncgrid,length(plowerbarGrid));
Empirical.BFbands = nan(Ncgrid,length(plowerbarGrid),length(epsilonGridEmpirical));

for jindex=1:length(epsilonGridEmpirical)
    epsilon = epsilonGridEmpirical(jindex);
    
    load(sprintf('../output/BF_empirical_Nint%2.2d_Nc%2.2d_B%2.2d_covars%g_eps%2.4f.mat',...
    gridParams.Ngrid,Ncgrid,Nbootstrap,params.covariatesOn,epsilon));
    
    Empirical.BFvec = BFvec;
    
    Empirical.BFbands(:,:,jindex) = CI.BFvec_CIuniformUnequal_onesided_lb;

end

clearvars -except alpha Empirical cgrid cupperbarNatural Ncgrid gridParams params plowerbarGrid Nbootstrap epsilonGridEmpirical;

%% Plots

% See note at beginning of file
alphaVal = 0.05;

printDirectory = sprintf('../images/Empirical_BFallBands_alpha%2.4g',alphaVal);
% The final piece of the file name is provided in the plot function below

% This is the band we want to single out as our `preferred' band
% This comes from our empirically calibrated simulation / bootstrap
% Allows the band to be different for each plowerbar
%
% For this vector you need to look at the coverage probabilities manually
% and order them appropriately
specialIndex = [1 6 7 6 4];

cbar_hhSize = 0.0403;
cbar_age = 0.0625;

plotOptions.minimal = 0;
plotEstVsTruth_empirical(Empirical.BFvec(:,1:3),Empirical.BFbands(:,1:3,:),cgrid,0.2,plowerbarGrid(1),plotOptions,specialIndex(1),printDirectory);

plotOptions.minimal = 2;
plotEstVsTruth_empirical(Empirical.BFvec(:,2:3),Empirical.BFbands(:,2:3,:),cgrid,0.2,plowerbarGrid(2:3),plotOptions,specialIndex(2:3),printDirectory);

%% Compute secant lines, as reported in the main text

plowerbarIndex = 1;
c2 = 7; % cgrid(7) = 0.0625
c1 = 5; % cgrid(5) = 0.0417

num2(plowerbarIndex) = Empirical.BFvec(c2,plowerbarIndex);
num1(plowerbarIndex) = Empirical.BFvec(c1,plowerbarIndex);

secant(plowerbarIndex) = (num2(plowerbarIndex) - num1(plowerbarIndex))/(cgrid(c2) - cgrid(c1));

end