function compileAllMCresults(dgpChoice)

MCgrids = setOverallSimGrids();

% These are already in the MCresults files, but we need them to initialize
% the vectors below
alphaGrid = MCgrids.alphaGrid;
plowerbarGrid = MCgrids.plowerbarGrid;

sampleSizeGrid = MCgrids.sampleSizeGrid;

coverageProbs = nan(length(MCgrids.epsilonGrid),length(plowerbarGrid),length(sampleSizeGrid),length(alphaGrid));
areaRatios = nan(length(MCgrids.epsilonGrid),length(plowerbarGrid),length(sampleSizeGrid));
MCmedian_ISE = nan(length(sampleSizeGrid),length(plowerbarGrid));

for i=1:length(sampleSizeGrid)
    sampleSize = sampleSizeGrid(i);
    
    for j=1:size(MCgrids.epsilonGrid,1)
        epsilon = MCgrids.epsilonGrid(j,i);

    % load MC results file for this specification
    try
        
    load(sprintf('../output/MCresults_dgp%d_n%d_eps%2.4f.mat',...
        dgpChoice,sampleSize,epsilon));
    % i is stored in this dataset. That's the problem here. Need to clear
    % it before this dataset is saved
    %
    % Note: Some of these files load their own epsilon grid.
    % So we switch to using MCgrids.epsilonGrid everywhere here
        
    % Get coverage probabilities
    for k=1:length(alphaGrid)
        coverageProbs(j,:,i,k) = UnifCovgProb_unequal(k,:);
        areaRatios(j,:,i,k) = meanRatioOfAreas(:,k)';
    end
    
    % Doesn't change with epsilon, so should be the same regardless of
    % which MCresults file we load. Just pick the first one.
    if j==1
        MCmedian_ISE(i,:) = median_ISE;
    end
    
    catch
        display(sprintf('Did not load MC results for dgp%d, sample size %d, and epsilon = %2.4f.',...
            dgpChoice,sampleSize,epsilon));
    end
    
    end
end

clear i j k;

%% Save overall MC results file, for use in printMCresults.m

save(sprintf('../output/MCresultsAll_dgp%d.mat',dgpChoice),'areaRatios','coverageProbs','median_ISE','MCgrids');

end