function cstarGrid = computeATEbp(mugrid,params,cgrid,Data,cdfBandwidth,DataBootstrap,epsilon)

cstarGrid = nan(length(mugrid),1);

options = optimset('TolX',1e-4,'TolFun',1e-4);
        
for i=1:length(mugrid)
    mu = mugrid(i);

    if nargin == 3
        fun = @(c) computeATElb(params,c) - mu;
    elseif nargin == 5
        fun = @(c) computeATElb(params,c,Data,cdfBandwidth) - mu;
    elseif nargin == 7
        fun = @(c) computeATElb(params,c,Data,cdfBandwidth,DataBootstrap,epsilon) - mu;
    end
    
    if nargin < 7 % Not the bootstrap case
    
    if fun(0) > 0
    
        cstarGrid(i) = fzero(fun, [0 max(cgrid)],options);

    else
        
        % Indicator for "there does not exist a breakdown point"
        % Zero isn't really correct to use, BUT the problem is that, when
        % running the bootstrap, sometimes there doesn't exist a breakdown
        % point, and hence these cases get mixed in with the cases where it
        % does exist, and that causes problems. So setting the BP to be
        % zero in that case is OK.
        cstarGrid(i) = 0;
        display(sprintf('For mu = %4.2f the breakdown point does not exist.',mu));
        
    end
    
    elseif nargin == 7 % Bootstrap case
        
        % Approach 1 is more robust than approach 2. But it's also about 3
        % times as slow.
        
        % APPROACH 1:
        %cStartValue = 0.25;
        %cstarGrid(i) = fzero(fun,cStartValue,options); 
        
        % APPROACH 2:
        cstarGrid(i) = fzero(fun,[-0.5,max(cgrid)],options);
        
        % This code allows for negative c's, which is important when
        % running the bootstrap, because we might have bootstrap samples
        % where the breakdown point doesn't exist, even if it does exist in
        % the main sample. This happens when the estimated cStar is very
        % close to zero.
        
    end
        
end

end