% This code computes the breakdown curve, which is the untruncated
% breakdown frontier. (Unless trunc.top = 1 or trunc.bottom = 1, in which
% case we truncate the BC.)
function BC = computeBC(intPieceVec,minPieceVec,plowerbar,trunc)

    Ncgrid = length(intPieceVec);

    BC = zeros(Ncgrid,1);
    for i=1:Ncgrid

        numer = plowerbar + intPieceVec(i) - 1;
        denom = intPieceVec(i) - minPieceVec(i) - 1;

        BC(i) = numer/denom;

    end
    
    % Note: Unless gridParams.Ngrid is large enough, intPiece and minPiece 
    % aren't accurate enough for large values of c and so BC can become 
    % very large (larger than 1). That is, the BC computed above will be
    % non-monotonic.
    %
    % Our main purpose of computing the BC is to allow for it to take
    % values larger than 1---for small values of c---so that we can use
    % this to define some of our confidence bands. Hence we next force BC
    % to be monotonic, and then truncate it as desired. Keep in mind that
    % unless gridParams.Ngrid is large enough, the negative portion of BC
    % will not be accurately computed, and should not be used in any 
    % procedures.

    for i=1:(Ncgrid-1)
        if BC(i+1) > BC(i)
            BC(i+1) = BC(i);
        end
    end
    
    if nargin == 4
        if trunc.top == 1
            for i=1:Ncgrid
                if BC(i) > 1
                    BC(i) = 1;
                end
            end
        end

        if trunc.bottom == 1
            for i=1:Ncgrid
                if BC(i) < 0
                    BC(i) = 0;
                end
            end
        end
    end
      
end