% Here I output the intPiece and minPiece vectors only for debugging and
% testing; they are not used anywhere else. They are only used for 
% computing BFvec here.
function [BFvec,intPieceVec,minPieceVec,minPieceVecNotMono] = ...
    computeBFvec(cgrid,plowerbarGrid,params,...
        gridParams,z,Data,cdfBandwidth,DataBootstrap,epsilon)

intPieceVec = nan(length(cgrid),1);
minPieceVec = nan(length(cgrid),1);

%% Compute the two components to the BF
for i=1:length(cgrid)
    
    c = cgrid(i);
    fprintf(1,'c = %d\n',i);
    
    % intPieceVec and minPieceVec are monotonic in c.
    % Hence we speed up computation here by skipping it once we hit
    % the boundary (0 or 1).
    
    if (i >= 2) && (intPieceVec(i-1) == 1)
        intPieceVec(i) = 1;
    else
    
        if nargin == 5
            intPieceVec(i) = UBintegralPiece(params,gridParams,z,c);

        elseif nargin == 7
            intPieceVec(i) = ...
                UBintegralPiece(params,gridParams,z,c,Data,cdfBandwidth);

        elseif nargin == 9
            intPieceVec(i) = ...
                UBintegralPiece(params,gridParams,z,c,...
                    Data,cdfBandwidth,DataBootstrap,epsilon);

        end    
    
    end
    
    if (i >= 2) && (minPieceVec(i-1) == 0)
        minPieceVec(i) = 0;
    else
        
        if nargin == 5
            minPieceVec(i) = UBminpiece(params,z,c);

        elseif nargin == 7
            minPieceVec(i) = UBminpiece(params,z,c,Data,cdfBandwidth);

        elseif nargin == 9
            minPieceVec(i) = UBminpiece(params,z,c,...
                    Data,cdfBandwidth,DataBootstrap,epsilon);

        end
    
    end
    
end

%% Error checking and correcting
% UBminpiece must be monotonically increasing in c. When UBobjfun is
% very jagged it is possible to get non-monotonicities. This typically
% happens only in bootstrap samples when the perturbed cdf is far from
% the empirical cdf.
%
% Increasing the number of grid search points does not seem to help.
% Instead, here I force monotonicity on UBminpiece. I remove the
% non-monotonic points and replace them with a linear interpolation of
% the monotonic points.

% Store the original vector, just for debugging
minPieceVecNotMono = minPieceVec;

minPieceVec = computeLinearMonoInterp(cgrid,minPieceVec);

%% Construct the BF for each claim of interest
BFvec = nan(length(cgrid),length(plowerbarGrid));

for j=1:length(plowerbarGrid)
    plowerbar = plowerbarGrid(j);

    BFvec(:,j) = computeBF(intPieceVec,minPieceVec,plowerbar);

end

end