function [CI,ATEbpBootCentered,ATEbpBoot] = computeBootstrapCI_ATEbp(mugrid,params,cgrid,Data,...
    ATEbreakdownPoints,cdfBandwidth,Nbootstrap,epsilon,alpha)

ATEbpBoot = nan(length(mugrid),Nbootstrap);
ATEbpBootCentered = nan(length(mugrid),Nbootstrap);

for b=1:Nbootstrap
    fprintf(1,'b = %d\n',b);
    
    if params.covariatesOn == 0

        % Generate bootstrap data
        databoot = datasample([Data.Y Data.X], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);

        % Here we compute and store all perturbed cdfs F_{Y|X} using the
        % bootstrap data. This speeds up computations later on because we can
        % just reference this cdf each time rather than recomputing it
        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,[],Data,DataBootstrap,epsilon);       
    
    elseif params.covariatesOn == 1

        % Generate bootstrap data
        databoot = datasample([Data.Y Data.X Data.Wsingle], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);
        DataBootstrap.Wsingle = databoot(:,3);

        % Here we compute and store all perturbed cdfs F_{Y|X} using the
        % bootstrap data. This speeds up computations later on because we can
        % just reference this cdf each time rather than recomputing it
        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,1,Data,DataBootstrap,epsilon);  
        
        DataBootstrap.cgrid = cgrid;
        DataBootstrap = storeFbounds(params,cgrid,cdfBandwidth,Data,DataBootstrap,epsilon);
        
        for i=1:Data.WsingleSuppSize
            WsuppPoint = i;

            params.p1hatCondStar(i) = ...
                mean(DataBootstrap.X(DataBootstrap.Wsingle == WsuppPoint));
            
            params.p0hatCondStar(i) = 1 - params.p1hatCond(i);
            
            params.p1cond(i) = params.p1hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p1hatCondStar(i) - params.p1hatCond(i));

            params.p0cond(i) = params.p0hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p0hatCondStar(i) - params.p0hatCond(i));

        end

        clear WsuppPoint i;
    
    end
    
    for i=1:length(mugrid)
        mu = mugrid(i);

        % Computes phi[thetaHat + epsilon*(thetaHat^* - thetaHat)]
        ATEbpBoot(i,b) = computeATEbp(mu,params,cgrid,Data,cdfBandwidth,DataBootstrap,epsilon);

        % The distribution of this across b will approximate the
        % distribution of sqrt(n) [ phi(thetaHat) - phi(thetaTrue) ]
        ATEbpBootCentered(i,b) = (ATEbpBoot(i,b) - ATEbreakdownPoints(i))/epsilon;

    end

end

CI = nan(length(mugrid),1);

for i=1:length(mugrid)
    
    % One sided CI
    % Recall that the *upper* quantile of sqrt(n) [ thetaHat - theta ] will
    % give us the *lower* bound for a CI for theta
    
    CenteredUB = quantile(ATEbpBootCentered(i,:), 1-alpha);
    CI(i) = ATEbreakdownPoints(i) - CenteredUB/sqrt(params.sampleSize);

end
    
end