function CI = computeBootstrapCI_ptIdentP1g0(params,...
    gridParams,z,c,Data,cdfBandwidth,Nbootstrap,alpha,WsingleSuppSize,baselineIsConditionalRankInvar)

paramBoot = nan(Nbootstrap,1);
paramBootWcell = nan(Nbootstrap,WsingleSuppSize);

% Naive bootstrap
epsilon = 1/sqrt(params.sampleSize);

for b=1:Nbootstrap
    fprintf(1,'b = %d\n',b);
    
    if params.covariatesOn == 0

        % Generate bootstrap data
        databoot = datasample([Data.Y Data.X], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);
        
        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,[],Data,DataBootstrap,epsilon);  

    elseif params.covariatesOn == 1
 
        % Generate bootstrap data
        databoot = datasample([Data.Y Data.X Data.Wsingle], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);
        DataBootstrap.Wsingle = databoot(:,3);
        
        DataBootstrap.WsingleSuppSize = WsingleSuppSize;

        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,1,Data,DataBootstrap,epsilon);  
        
        cgrid = [0]; % Only ever need c=0 here
        DataBootstrap.cgrid = cgrid;
        DataBootstrap = storeFbounds(params,cgrid,cdfBandwidth,Data,DataBootstrap,epsilon);
        
        for i=1:Data.WsingleSuppSize
            WsuppPoint = i;

            params.p1hatCondStar(i) = ...
                mean(DataBootstrap.X(DataBootstrap.Wsingle == WsuppPoint));
            
            params.p0hatCondStar(i) = 1 - params.p1hatCond(i);
            
            params.p1cond(i) = params.p1hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p1hatCondStar(i) - params.p1hatCond(i));

            params.p0cond(i) = params.p0hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p0hatCondStar(i) - params.p0hatCond(i));

        end

        clear WsuppPoint i;
        
    end
    
    if baselineIsConditionalRankInvar == 0
    
    paramBoot(b) = ...
        1-UBintegralPiece(params,gridParams,z,c,DataBootstrap,cdfBandwidth,DataBootstrap,epsilon);
    
    elseif baselineIsConditionalRankInvar == 1

    for i=1:Data.WsingleSuppSize
        WsuppPoint = i;

        params.pWstar(WsuppPoint) = mean(DataBootstrap.Wsingle==WsuppPoint);

        % Define new cdfBandwidth
        cdfBandwidthWcell.x1 = 0;
        cdfBandwidthWcell.x0 = 0;

        % Define new params
        paramsWcell.covariatesOn = 0;
        paramsWcell.storeFbounds = 1;
        paramsWcell.sampleSize = length(DataBootstrap.Wsingle);

        % Prop score in bootstrap sample
        paramsWcell.p1hatStar = params.p1hatCondStar(WsuppPoint);
        paramsWcell.p0hatStar = 1 - paramsWcell.p1hatStar;

        % Perturbed estimator of prop score
        paramsWcell.p1 = params.p1cond(WsuppPoint);
        paramsWcell.p0 = params.p0cond(WsuppPoint);

        % Original sample prop scores
        paramsWcell.p1hat = params.p1hatCond(WsuppPoint);
        paramsWcell.p0hat = params.p0hatCond(WsuppPoint);

        DataBootstrapWcell.covariatesOn = 0;
        
        DataBootstrapWcell.perturbedCDFflag = DataBootstrap.perturbedCDFflag;
        DataBootstrapWcell.Ymin = DataBootstrap.Ymin;
        DataBootstrapWcell.Ymax = DataBootstrap.Ymax;
        DataBootstrapWcell.ygrid = DataBootstrap.ygrid;
        DataBootstrapWcell.cgrid = DataBootstrap.cgrid;

        DataBootstrapWcell.Y = DataBootstrap.Y(DataBootstrap.Wsingle==WsuppPoint);
        DataBootstrapWcell.X = DataBootstrap.X(DataBootstrap.Wsingle==WsuppPoint);

        DataBootstrapWcell.PerturbedFhatMonotonicVec = DataBootstrap.PerturbedFhatMonotonicVec(:,:,WsuppPoint);
        DataBootstrapWcell.Flower = DataBootstrap.FlowerCondW(:,:,:,WsuppPoint);
        DataBootstrapWcell.Fupper = DataBootstrap.FupperCondW(:,:,:,WsuppPoint);
            
        paramBootWcell(b,i) = ...
            1-UBintegralPiece(paramsWcell,gridParams,z,c,DataBootstrapWcell,cdfBandwidthWcell,DataBootstrapWcell,epsilon);

    end
    clear WsuppPoint i;

    paramBoot(b) = paramBootWcell(b,:) * params.pWstar';

    end
end

% Two sided CI
CI.lb = quantile(paramBoot, alpha/2);
CI.ub = quantile(paramBoot, 1-alpha/2);

% One sided lower CI
CI.OneSidedLower = quantile(paramBoot, alpha);
    
end