% Note: A bunch of vectors are outputted solely for debugging
function [BFvecBootCentered, BFvecBoot, BCvecBoot, BCvecBootCentered,...
    intPieceVecBoot, minPieceVecBoot,minPieceVecBootNotMono] = computeBootstrapVecs(params,...
    gridParams,z,cdfBandwidth,cgrid,plowerbarGrid,Data,BFvec,BCvec,...
    Nbootstrap,epsilon,trunc,baselineIsConditionalRankInvar)
%% Bootstrap
    
intPieceVecBoot = nan(length(cgrid),Nbootstrap);
minPieceVecBoot = nan(length(cgrid),Nbootstrap);
minPieceVecBootNotMono = nan(length(cgrid),Nbootstrap);

intPieceVecBootWcell = nan(length(cgrid),Nbootstrap,Data.WsingleSuppSize);
minPieceVecBootWcell = nan(length(cgrid),Nbootstrap,Data.WsingleSuppSize);

BFvecBoot = nan(length(cgrid),length(plowerbarGrid),Nbootstrap);
BFvecBootCentered = nan(length(cgrid),length(plowerbarGrid),Nbootstrap);
BCvecBoot = nan(length(cgrid),length(plowerbarGrid),Nbootstrap);
BCvecBootCentered = nan(length(cgrid),length(plowerbarGrid),Nbootstrap);
    
for b=1:Nbootstrap
    fprintf(1,'b = %d\n',b);
    
    if Data.covariatesOn == 0

        % Generate bootstrap data
        databoot = datasample([Data.Y Data.X], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);

        % Here we compute and store all perturbed cdfs F_{Y|X} using the
        % bootstrap data. This speeds up computations later on because we can
        % just reference this cdf each time rather than recomputing it
        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,[],Data,DataBootstrap,epsilon);

        % Bootstrap treatment probability estimate
        params.p1hatStar = mean(DataBootstrap.X); 
        params.p0hatStar = 1 - params.p1hatStar;

        % Perturbed estimator
        params.p1 = params.p1hat + ...
            epsilon*sqrt(params.sampleSize)*(params.p1hatStar - params.p1hat);

        params.p0 = params.p0hat + ...
            epsilon*sqrt(params.sampleSize)*(params.p0hatStar - params.p0hat);
        
        % Large epsilons can lead to probabilities < 0 and > 1, although it
        % is rare. In these cases, we truncate the probabilities to lie in
        % (0,1).
        if params.p1 < 0
            params.p1 = 0.001;
        elseif params.p1 > 1
            params.p1 = 0.999;
        end
        
        if params.p0 < 0
            params.p0 = 0.001;
        elseif params.p0 > 1
            params.p0 = 0.999;
        end

    elseif Data.covariatesOn == 1

        databoot = datasample([Data.Y Data.X Data.Wsingle], params.sampleSize);
        DataBootstrap.Y = databoot(:,1);
        DataBootstrap.X = databoot(:,2);
        DataBootstrap.Wsingle = databoot(:,3);
        
        DataBootstrap.perturbedCDFflag = 0;
        [~,DataBootstrap] = computePerturbedCDF(0,0,1,Data,DataBootstrap,epsilon);

        DataBootstrap.cgrid = cgrid;
        DataBootstrap = storeFbounds(params,cgrid,cdfBandwidth,Data,DataBootstrap,epsilon);

        for i=1:Data.WsingleSuppSize
            WsuppPoint = i;

            params.p1hatCondStar(i) = ...
                mean(DataBootstrap.X(DataBootstrap.Wsingle == WsuppPoint));
            
            params.p0hatCondStar(i) = 1 - params.p1hatCondStar(i);
            
            params.p1cond(i) = params.p1hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p1hatCondStar(i) - params.p1hatCond(i));

            params.p0cond(i) = params.p0hatCond(i) + ...
                epsilon*sqrt(params.sampleSize)*(params.p0hatCondStar(i) - params.p0hatCond(i));

        end

        clear WsuppPoint i;
    end
        
    if baselineIsConditionalRankInvar == 1
        
        for i=1:Data.WsingleSuppSize
            WsuppPoint = i;
            
            params.pWstar(WsuppPoint) = mean(DataBootstrap.Wsingle==WsuppPoint);

            % Define new "Data" variable for a single covariate cell
            DataWcell.covariatesOn = 0;
            DataWcell.Y = Data.Y(Data.Wsingle==WsuppPoint);
            DataWcell.X = Data.X(Data.Wsingle==WsuppPoint);

            % Define new cdfBandwidth
            cdfBandwidthWcell.x1 = 0;
            cdfBandwidthWcell.x0 = 0;

            % Define new params
            paramsWcell.covariatesOn = 0;
            paramsWcell.storeFbounds = 1;
            paramsWcell.sampleSize = length(DataWcell.Y);
            
            % Prop score in bootstrap sample
            paramsWcell.p1hatStar = params.p1hatCondStar(WsuppPoint);
            paramsWcell.p0hatStar = 1 - paramsWcell.p1hatStar;

            % Perturbed estimator of prop score
            paramsWcell.p1 = params.p1cond(WsuppPoint);
            paramsWcell.p0 = params.p0cond(WsuppPoint);
            
            % Original sample prop scores
            paramsWcell.p1hat = params.p1hatCond(WsuppPoint);
            paramsWcell.p0hat = params.p0hatCond(WsuppPoint);

            DataBootstrapWcell.perturbedCDFflag = 1;
            DataBootstrapWcell.Ymin = DataBootstrap.Ymin;
            DataBootstrapWcell.Ymax = DataBootstrap.Ymax;
            DataBootstrapWcell.ygrid = DataBootstrap.ygrid;
            DataBootstrapWcell.cgrid = DataBootstrap.cgrid;
            
            DataBootstrapWcell.Y = DataBootstrap.Y(DataBootstrap.Wsingle==WsuppPoint);
            DataBootstrapWcell.X = DataBootstrap.X(DataBootstrap.Wsingle==WsuppPoint);
            
            DataBootstrapWcell.PerturbedFhatMonotonicVec = DataBootstrap.PerturbedFhatMonotonicVec(:,:,WsuppPoint);
            DataBootstrapWcell.Flower = DataBootstrap.FlowerCondW(:,:,:,WsuppPoint);
            DataBootstrapWcell.Fupper = DataBootstrap.FupperCondW(:,:,:,WsuppPoint);
            
            [~,intPieceVecBootWcell(:,b,i),minPieceVecBootWcell(:,b,i),~] = ...
                computeBFvec(cgrid,plowerbarGrid,paramsWcell,...
                    gridParams,z,DataWcell,cdfBandwidthWcell,DataBootstrapWcell,epsilon);
            
        end
        clear WsuppPoint i;
        
        intPieceVecBoot(:,b) = squeeze(intPieceVecBootWcell(:,b,:)) * params.pWstar';
        minPieceVecBoot(:,b) = squeeze(minPieceVecBootWcell(:,b,:)) * params.pWstar';

        trunc.top = 1;
        trunc.bottom = 1;
        BFvecBoot(:,:,b) = computeBCvec(intPieceVecBoot(:,b),minPieceVecBoot(:,b),plowerbarGrid,trunc);

        % BF estimates
        BFvecBootCentered(:,:,b) = (BFvecBoot(:,:,b) - BFvec)/epsilon;

        % BC estimates
        BCvecBoot(:,:,b) = ...
         computeBCvec(...
            intPieceVecBoot(:,b),minPieceVecBoot(:,b),plowerbarGrid,trunc);

        BCvecBootCentered(:,:,b) = (BCvecBoot(:,:,b) - BCvec)/epsilon;
        
    elseif baselineIsConditionalRankInvar ==0
        
        % BF estimates
        [BFvecBoot(:,:,b),intPieceVecBoot(:,b),minPieceVecBoot(:,b),minPieceVecBootNotMono(:,b)] = ...
            computeBFvec(cgrid,plowerbarGrid,params,...
                    gridParams,z,Data,cdfBandwidth,DataBootstrap,epsilon);

        BFvecBootCentered(:,:,b) = (BFvecBoot(:,:,b) - BFvec)/epsilon;

        % BC estimates
        BCvecBoot(:,:,b) = ...
         computeBCvec(...
            intPieceVecBoot(:,b),minPieceVecBoot(:,b),plowerbarGrid,trunc);

        BCvecBootCentered(:,:,b) = (BCvecBoot(:,:,b) - BCvec)/epsilon;

    end
    
end

end
