function computeEmpiricalMCresults(nSimuls,epsilonToAggregate)
%% Load population dataset and grid parameters

toLoad.Nint = 100;
toLoad.Ncgrid = 20;
toLoad.B = 1000;
toLoad.covariatesOn = 1;
toLoad.epsilon = 0.0236; % This choice doesn't matter since we are only
                         % going to extract BFvec.

load(sprintf('../output/BF_empirical_Nint%2.2d_Nc%2.2d_B%2.2d_covars%g_eps%2.4f.mat',...
    toLoad.Nint,toLoad.Ncgrid,toLoad.B,toLoad.covariatesOn,toLoad.epsilon));

clear toLoad;

BFvecTruth = BFvec;

clearvars -except nSimuls epsilonToAggregate BFvecTruth cgrid plowerbarGrid;

%% Load MC results

MC.BFvec = nan(length(cgrid),length(plowerbarGrid),nSimuls);

% WARNING: This vector needs to be manually adjusted to account for the
% size of alphaGrid
numberOfAlphas = 2;
MC.BFvec_CIuniformUnequal_onesided_lb = nan(length(cgrid),length(plowerbarGrid),nSimuls,numberOfAlphas);
clear numberOfAlphas;

MC.simLoaded = nan(nSimuls,1);

for isim=1:nSimuls
    simulationNumber = isim;

    try

        load(sprintf('../MCestimatesEmpirical/BF_EmpiricalMCestimates_eps%2.4f_sim%d.mat',...
            epsilonToAggregate,simulationNumber));
        
        % Used for computing the ratio of area under band to area under BF
        MC.BFvec(:,:,simulationNumber) = BFvec;
        
        for k=1:length(alphaGrid)

            MC.BFvec_CIuniformUnequal_onesided_lb(:,:,simulationNumber,k) = ...
                CI(k).BFvec_CIuniformUnequal_onesided_lb;
        
        end
                
        MC.simLoaded(isim) = 1;
        
    catch
        display(sprintf('Did not load simulation dataset number %d.',...
            simulationNumber));
        
        MC.simLoaded(isim) = 0;
        
    end

end

clear isim;

%% Compute uniform coverage probabilities

for k=1:length(alphaGrid)
    alpha = alphaGrid(k);

    [UnifCovgProb_unequal(k,:),UnifCoverageIndic_unequal(:,:,k)] = ...
      computeUniformCoverageProb_lb(...
        MC.BFvec_CIuniformUnequal_onesided_lb(:,:,MC.simLoaded==1,k),BFvecTruth);

    fprintf(1,'nSimuls = %d, epsilon = %2.4f, alpha = %2.4g\n',...
        nSimuls,epsilonToAggregate,alpha);

    display(UnifCovgProb_unequal(k,:));

end

%% Plot ALL MC estimates at once

plotOptions.minimal = 0;
plotOptions.reverseTruthColor = 0;

printDirectory = sprintf('../images/MC_BF_Empirical_allEstimates');
% The final piece of the file name is provided in the plot function below

plotEstVsTruth(BFvecTruth,MC.BFvec,cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

%% Plot ALL confidence bands at once

for k=1:length(alphaGrid)
    alpha = alphaGrid(k);

printDirectory = sprintf('../images/MCempirical_BFallBand_alpha%2.4g_epsilon%2.4g',...
    alpha,epsilon);
% The final piece of the file name is provided in the plot function below

plotEstVsTruth(BFvecTruth,MC.BFvec_CIuniformUnequal_onesided_lb(:,:,:,k),cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

end

%% Save results
clear i j k job_id job_id_string printDirectory;

save(sprintf('../output/MCresultsEmpirical_eps%2.4f.mat',epsilonToAggregate));

end