% Inputs: a grid of points in the domain and a grid of function values
% This function then takes that function and makes it weakly increasing. It
% does this with linear interpolation
function minPieceVec = computeLinearMonoInterp(cgrid,minPieceVec)

% First, find the indices of all c values where we have a non-monotonicity
% We want to get rid of those values and replace them with interpolations
minPieceVecFlatMono = minPieceVec;

for i=1:(length(cgrid)-1)
    if minPieceVecFlatMono(i+1) < minPieceVecFlatMono(i)
        minPieceVecFlatMono(i+1) = minPieceVecFlatMono(i);
    end
end
% We could stop here. But I suspect that a linear interpolation will work
% better, given the usual shape of the minPieceVec

nonMonotonic = (minPieceVecFlatMono ~= minPieceVec);

minPieceVecLinearMono = minPieceVec;

lastPointIncluded = 0;

% Only modify minPieceVec if there are non-monotonic regions
if sum(nonMonotonic) > 0
    i = 1;
    while i <= length(cgrid)
        
        if nonMonotonic(i) == 1
            
            % Case 1: Only the last coordinate is non-monotonic
            if i == length(cgrid)
                minPieceVecLinearMono(i) = minPieceVec(i-1);
                break;
            end

            % Case 2: Either
            % (A) there are two or more non-monotonic points, or
            % (B) there is one non-monotonic point, but it is not the last one            
            
            % Our next goal is to see if there is a sequence of
            % non-monotonic points beginning at i. If so, we want to find
            % where it ends
            j = i+1;
            while nonMonotonic(j) == 1

                % Stop if there is a sequence of non-monotonic points which
                % ends at the last point
                if j == length(cgrid)
                    lastPointIncluded = 1;
                    break;
                end
                
                j = j + 1;
                
            end
            
            % Now, [c(i) c(j-1)] is a string of grid points where we need to
            % replace their values, using linear interpolation from c(i-1)
            % to c(j)
            
            % Current sequence does not end at the last grid point
            if lastPointIncluded == 0
            
            minPieceVecLinearMono(i:(j-1)) = ...
                interp1([cgrid(i-1) cgrid(j)],...
                [minPieceVec(i-1) minPieceVec(j)],cgrid(i:(j-1)));
                        
            i = j;
            
            % Current sequence ends at the last grid point
            elseif lastPointIncluded == 1
                for k=i:j
                    minPieceVecLinearMono(k) = minPieceVec(i-1);
                end
                
                break; % Because in this case we're done!
                
            end
            
        else
            % Turns out the i-th grid point was fine---it was monotonic.
            % So return to the beginning and check the next one
            i = i+1;
            
        end
    end    
    
    minPieceVec = minPieceVecLinearMono;
    
end

end