function computeMCresults(dgpChoice,sampleSize,nSimuls,epsilon)
%% Load population dataset and grid parameters
load(sprintf('../output/BF_MCtruth_dgp%d.mat',dgpChoice));

BFvecTruth = BFvec;

%% Load MC results

MC.BFvec = nan(length(cgrid),length(plowerbarGrid),nSimuls);

MC.BCvec = nan(length(cgrid),length(plowerbarGrid),nSimuls);

% WARNING: This vector needs to be manually adjusted to account for the
% size of alphaGrid
numberOfAlphas = 1;
MC.BFvec_CIuniformUnequal_onesided_lb = nan(length(cgrid),length(plowerbarGrid),nSimuls,numberOfAlphas);
clear numberOfAlphas;

MC.simLoaded = nan(nSimuls,1);

for isim=1:nSimuls
    simulationNumber = isim;

    try
        
        % This dataset has a leaked i variable in it. For this reason
        % I used the index isim here.
        load(sprintf('../MCestimates/BF_MCestimates_dgp%d_n%d_eps%2.4f_sim%d.mat',...
            dgpChoice,sampleSize,epsilon,simulationNumber));
        % This dataset may also load a variable nSimulations. This is
        % different from nSimuls. nSimuls has also leaked somewhat.
        % For this reason, I use simulationNumber here.

        MC.BFvec(:,:,simulationNumber) = BFvec;
        
        MC.BCvec(:,:,simulationNumber) = BCvec;
        
        for k=1:length(alphaGrid)

            MC.BFvec_CIuniformUnequal_onesided_lb(:,:,simulationNumber,k) = ...
                CI(k).BFvec_CIuniformUnequal_onesided_lb;
        
        end
                
        MC.simLoaded(isim) = 1;
        
    catch
        display(sprintf('Did not load simulation dataset number %d.',...
            simulationNumber));
        
        MC.simLoaded(isim) = 0;
        
        % Note: The bias and standard deviation computations below require
        % all simulations to be loaded.
        
    end

end

clear isim;

%% Compute uniform coverage probabilities
% Uniform coverge says that, for a fixed plowerbar, we cover for all values
% of c. Thus, for a single simulation, we just need to check whether
% the entire column of UnifPtCoverageIndic is 1.

for k=1:length(alphaGrid)
    alpha = alphaGrid(k);

    [UnifCovgProb_unequal(k,:),UnifCoverageIndic_unequal(:,:,k)] = ...
      computeUniformCoverageProb_lb(...
        MC.BFvec_CIuniformUnequal_onesided_lb(:,:,MC.simLoaded==1,k),BFvecTruth);

    fprintf(1,'dgp = %d, sampleSize = %d, nSimuls = %d, epsilon = %2.4f, alpha = %2.4g\n',...
        dgpChoice,sampleSize,nSimuls,epsilon,alpha);

    display(UnifCovgProb_unequal(k,:));

end

%% Compute estimator bias (pointwise in c)

% The bias is the difference between these vectors and the truth
% Easier to just plot these vectors against the truth, though, than to look
% at the bias directly
BFhat_mean = mean(MC.BFvec(:,:,MC.simLoaded==1),3);
BFhat_median = median(MC.BFvec(:,:,MC.simLoaded==1),3);

plotOptions.reverseTruthColor = 0;
plotOptions.minimal = 1;

% Plot the mean BF versus the truth. This shows us the mean bias visually
printDirectory = sprintf('../images/MC_BFbias_dgp%d_n%d',...
    dgpChoice,sampleSize);

plotEstVsTruth(BFvecTruth,BFhat_mean,cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

% Plot the median BF versus the truth. This shows us the median bias
% visually
printDirectory = sprintf('../images/MC_BFmedianBias_dgp%d_n%d',...
    dgpChoice,sampleSize);

plotEstVsTruth(BFvecTruth,BFhat_median,cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

% Plot the BF stderr versus the truth. This shows us the variance visually
printDirectory = sprintf('../images/MC_BFstderr_dgp%d_n%d',...
    dgpChoice,sampleSize);

plotOptions.minimal = 0;

%% Compute estimator MISE

BF_ISEvec = nan(nSimuls,length(plowerbarGrid));
cstep = (cgrid(end)-0)/length(cgrid);

for i=1:nSimuls
    
    BFcurrent = MC.BFvec(:,:,i);
    
    squaredDistance = (BFcurrent - BFvecTruth).^2;
    BF_ISEvec(i,:) = sum(squaredDistance,1)*cstep;
    
end

MISE = mean(BF_ISEvec(MC.simLoaded==1,:),1);
median_ISE = median(BF_ISEvec(MC.simLoaded==1,:),1);

%% Find the simulation run with the median IMSE

val = nan(1,length(plowerbarGrid));
ind = nan(2,length(plowerbarGrid));
median_ISE_simRun = nan(1,length(plowerbarGrid));

for j=1:length(plowerbarGrid)
    
[val(j) ind(:,j)] = mymedian(BF_ISEvec(:,j));
median_ISE_simRun(j) = ind(1,j); 
                            % There may be two values closest to the 
                            % median, when there are an even number of
                            % simulations. I just arbitrarily take one of
                            % them.
                            
end

clear val;

%% Plot the estimate for the simulation at the median

BFvec_medianISE = nan(length(cgrid),length(plowerbarGrid));
BFvec_CI_medianISE = nan(length(cgrid),length(plowerbarGrid),length(alphaGrid));

for j=1:length(plowerbarGrid)
    
    BFvec_medianISE(:,j) = MC.BFvec(:,j,median_ISE_simRun(j));
    
    for k=1:length(alphaGrid)
        BFvec_CI_medianISE(:,j,k) = MC.BFvec_CIuniformUnequal_onesided_lb(:,j,median_ISE_simRun(j),k);
    end

end

for k=1:length(alphaGrid)
    alpha = alphaGrid(k);

printDirectory = sprintf('../images/MC_BFmedianISE_dgp%d_n%d_eps%2.4f_alpha%2.4f',...
    dgpChoice,sampleSize,epsilon,alpha);
% The final piece of the file name is provided in the plot function below

plotEstVsTruth_withLowerCIband(BFvecTruth,BFvec_medianISE,...
    [],BFvec_CI_medianISE(:,:,k),...
    cgrid,cupperbar,plowerbarGrid,printDirectory,median_ISE_simRun);

end

%% Plot ALL MC estimates at once
plotOptions.reverseTruthColor = 1;

printDirectory = sprintf('../images/MC_BFallEstimates_dgp%d_n%d',...
    dgpChoice,sampleSize);
% The final piece of the file name is provided in the plot function below

plotEstVsTruth(BFvecTruth,MC.BFvec,cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

plotOptions.reverseTruthColor = 0;
%% Plot ALL confidence bands at once

for k=1:length(alphaGrid)
    alpha = alphaGrid(k);

printDirectory = sprintf('../images/MC_BFallBands_dgp%d_n%d_alpha%2.4g_epsilon%2.4g',...
    dgpChoice,sampleSize,alpha,epsilon);
% The final piece of the file name is provided in the plot function below

plotEstVsTruth(BFvecTruth,MC.BFvec_CIuniformUnequal_onesided_lb(:,:,:,k),cgrid,cupperbar,plowerbarGrid,plotOptions,printDirectory);

end

%% Save results
clear i j k;
% These leaked earlier from somewhere.

clear job_id job_id_string printDirectory;
clear epsilonGrid;

save(sprintf('../output/MCresults_dgp%d_n%d_eps%2.4f.mat',...
    dgpChoice,sampleSize,epsilon));

end