% This function takes the original data, the bootstrap sample, and then
% computes the perturbed cdf estimator on a large grid of points. The key
% point is that this estimator is a valid cdf. Then, whenever we need to
% call this perturbed cdf, we just lookup its value on this grid
function [cdfValue,DataBootstrap] = computePerturbedCDF(y,xEval,wIndex,Data,DataBootstrap,epsilon)

Nygrid = 10000;

% In this case, we haven't computed the cdf yet, so do it
if DataBootstrap.perturbedCDFflag == 0

    xgrid = [0 1];
    
    Y = Data.Y;
    X = Data.X;

    Ystar = DataBootstrap.Y;
    Xstar = DataBootstrap.X;
    n = length(Ystar);
    
    DataBootstrap.Ymin = min(Y);
    DataBootstrap.Ymax = max(Y);
    
    ygrid = linspace(DataBootstrap.Ymin,DataBootstrap.Ymax,Nygrid);
    DataBootstrap.ygrid = ygrid;
    
    if Data.covariatesOn == 0
        
    PerturbedFhatMonotonicVec = nan(length(ygrid),length(xgrid));
    DataBootstrap.PerturbedFhatMonotonicVec = nan(length(ygrid),length(xgrid));
    
    for j=1:length(xgrid)
        x = xgrid(j);
 
        FhatVec = nan(length(ygrid),1);
        FhatstarVec = nan(length(ygrid),1);
        PerturbedFhatVec = nan(length(ygrid),1);
        
        for i=1:length(ygrid)
            y = ygrid(i);

            FhatVec(i) = sum( (Y <= y).*(X == x) )/ sum( (X==x) );
            FhatstarVec(i) = sum( (Ystar <= y).*(Xstar == x) )/ sum( (Xstar==x) );

            PerturbedFhatVec(i) = ...
                FhatVec(i) + epsilon*sqrt(n)*(FhatstarVec(i) - FhatVec(i));

        end

        % Making the perturbed cdf a valid cdf
        for i=1:length(ygrid)
            if PerturbedFhatVec(i) < 0
                PerturbedFhatMonotonicVec(i,j) = 0;
            elseif PerturbedFhatVec(i) > 1
                PerturbedFhatMonotonicVec(i,j) = 1;
            else
                PerturbedFhatMonotonicVec(i,j) = PerturbedFhatVec(i);
            end
        end

        for i=1:(length(ygrid)-1)
            if PerturbedFhatMonotonicVec(i+1,j) < PerturbedFhatMonotonicVec(i,j)
                PerturbedFhatMonotonicVec(i+1,j) = PerturbedFhatMonotonicVec(i,j);
            end
        end

        DataBootstrap.PerturbedFhatMonotonicVec(:,j) = PerturbedFhatMonotonicVec(:,j);

    end
    
    elseif Data.covariatesOn == 1
   
    Wsingle = Data.Wsingle;
    WsingleStar = DataBootstrap.Wsingle;
    
    PerturbedFhatMonotonicVec = nan(length(ygrid),length(xgrid),Data.WsingleSuppSize);
    DataBootstrap.PerturbedFhatMonotonicVec = nan(length(ygrid),length(xgrid),Data.WsingleSuppSize);
    
    for j=1:length(xgrid)
        x = xgrid(j);
        
        for k=1:Data.WsingleSuppSize
            WsuppPoint = k;

            FhatVec = nan(length(ygrid),1);
            FhatstarVec = nan(length(ygrid),1);
            PerturbedFhatVec = nan(length(ygrid),1);

            for i=1:length(ygrid)
                y = ygrid(i);
                
                indicator = (X == x).*(Wsingle == WsuppPoint);
                indicatorStar = (Xstar == x).*(WsingleStar == WsuppPoint);

                FhatVec(i) = sum( (Y <= y).*indicator )/ sum(indicator);
                FhatstarVec(i) = sum( (Ystar <= y).*indicatorStar )/ sum(indicatorStar);

                PerturbedFhatVec(i) = ...
                    FhatVec(i) + epsilon*sqrt(n)*(FhatstarVec(i) - FhatVec(i));

            end

            % Making the perturbed cdf a valid cdf
            for i=1:length(ygrid)
                if PerturbedFhatVec(i) < 0
                    PerturbedFhatMonotonicVec(i,j,k) = 0;
                elseif PerturbedFhatVec(i) > 1
                    PerturbedFhatMonotonicVec(i,j,k) = 1;
                else
                    PerturbedFhatMonotonicVec(i,j,k) = PerturbedFhatVec(i);
                end
            end

            for i=1:(length(ygrid)-1)
                if PerturbedFhatMonotonicVec(i+1,j,k) < PerturbedFhatMonotonicVec(i,j,k)
                    PerturbedFhatMonotonicVec(i+1,j,k) = PerturbedFhatMonotonicVec(i,j,k);
                end
            end

            DataBootstrap.PerturbedFhatMonotonicVec(:,j,k) = PerturbedFhatMonotonicVec(:,j,k);

        end
    
    end
    
    end
    
    % Turn on flag
    DataBootstrap.perturbedCDFflag = 1;
    
end

if xEval == 0
    xIndex = 1;
elseif xEval == 1
    xIndex = 2;
end

if Data.covariatesOn == 0
    CDFgivenX = DataBootstrap.PerturbedFhatMonotonicVec(:,xIndex);    
elseif Data.covariatesOn == 1 
    CDFgivenX = DataBootstrap.PerturbedFhatMonotonicVec(:,xIndex,wIndex);
end

if y < DataBootstrap.Ymin
    cdfValue = 0;
elseif  y > DataBootstrap.Ymax
    cdfValue = 1;
else

    DomainIsLarger = (DataBootstrap.ygrid >= y);
    DomainIndex = length(DataBootstrap.ygrid) - sum(DomainIsLarger) + 1;
    
    cdfValue = CDFgivenX(DomainIndex);
    
end

end