function plotEstVsTruth_empirical_MOD_FOR_COARSENINGS(BFtrueVec,BFhatVec,cgrid,cupperbar,plowerbarGrid,plotOptions,specialIndex,printDirectory)

if nargin == 8
    h = figure('visible','off');
else
    h = figure('visible','on');    
end

for j=1:length(plowerbarGrid)
    plowerbar = plowerbarGrid(j);

    BFtrue = BFtrueVec(:,j);

    %plot(cgrid,BFtrue,'-','LineWidth',3,'Color','black'); % Manually
    %switch between these two lines depending on which plots you want to
    %show (binary versus trinary coarsenings)
    plot(cgrid,BFtrue,'--','LineWidth',2.5,'Color','black');
    hold on;

    cbar_hhSize = 0.0403;
    cbar_age = 0.0625;
    set(gca, 'XTick', sort([cbar_hhSize, cbar_age, get(gca, 'XTick')]));
    xticklabels({0, [], 0.05, [], 0.1, 0.15, 0.2});
    
    for i=1:size(BFhatVec,3)
        
        BFhat = BFhatVec(:,j,i);
        
        if ndims(BFhatVec) == 3
                plot(cgrid,BFhat,'--','LineWidth',2.5,'Color','black');
        elseif ndims(BFhatVec) == 2
            plot(cgrid,BFhat,'-','LineWidth',1.5,'Color','black');
        end
        set(gca,'fontsize',15);
        
        if plotOptions.minimal == 0
            
            yticks([0 0.2 0.4 0.6 0.8 1]);
            xlabel('Relaxation of full independence');
            ylabel('Relaxation of rank invariance');            

        elseif plotOptions.minimal == 1 
            %set(gca,'fontsize',25); % Use this for tiny plots
            set(gca,'fontsize',15);
            
            yaxisStop = max([BFtrue(1) BFhat(1)]);
            if yaxisStop == 0
                error('Estimated BF is degenerate / Estimated robust region is empty');
            end
            
            xIntersectIndex = min(find(BFtrue == 0));
            xIntersect = cgrid(xIntersectIndex);
            
            set(gca,'Xtick',[xIntersect cupperbar],'Ytick',[0 yaxisStop 1]);
 
           
            xticklabels(round([xIntersect cupperbar],3));
            yticklabels(round([0 yaxisStop 1],3));
            
            % Inputs: [x1,x2],[y1,y2]. 
            % Then it draws a line from (x1,y1) to (x2,y2)

            line([0,0],[0,yaxisStop],'Color','black');
            
        elseif plotOptions.minimal == 2
           
            set(gca,'fontsize',15);
            
            yaxisStop = max([BFtrue(1) BFhat(1)]);
            if yaxisStop == 0
                error('Estimated BF is degenerate / Estimated robust region is empty');
            end
            
            xIntersectIndex = min(find(BFtrue == 0));
            xIntersect = cgrid(xIntersectIndex);
            
            if mod(xIntersect,0.05) ~= 0
                xticks([0:0.05:xIntersect xIntersect]);
                xticklabels(round([0:0.05:xIntersect xIntersect cupperbar],3));
            else
                xticks([0:0.05:xIntersect]);
                xticklabels(round([0:0.05:xIntersect cupperbar],3));
            end
            
            if mod(yaxisStop,0.2) ~= 0
                yticks([0:0.2:yaxisStop yaxisStop 1]);            
                yticklabels(round([0:0.2:yaxisStop yaxisStop 1],3));
            else
                yticks([0:0.2:yaxisStop 1]);            
                yticklabels(round([0:0.2:yaxisStop 1],3));            
            end

            line([0,0],[0,yaxisStop],'Color','black');          
            
            xlabel(' ');
        end
        
        box off;        
        axis([0,cupperbar,0,1]);

    end
    
    hold off;
                
    if nargin == 8
        fileNameEnd = sprintf('_plowerbar%2.2g.eps',plowerbar);
        
        set(gca, 'Color', 'None');
        saveas(gcf,strcat(printDirectory,fileNameEnd),'epsc')
        
    end

end

end