function printNaiveComparisons(Data)

Y = Data.Y;
X = Data.X;
W = Data.Woriginal;

% X=1 means NOT abducted
% X=0 means abducted

expYlabel = 'Daily wage in Uganda shillings';
Ylabel = 'Log wage';
Wlabels ={'Age when surveyed','Household size in 1996'};

%% Compute statistics

mean1.sampleSize = sum(X(X==1));
mean0.sampleSize = length(X)-sum(X(X==1));

% Means
mean1.expY = mean(exp(Y(X==1)));
mean0.expY = mean(exp(Y(X==0)));

mean1.Y = mean(Y(X==1));
mean0.Y = mean(Y(X==0));

mean1.W = mean(W(X==1,:));
mean0.W = mean(W(X==0,:));

% Standard errors
stdder1.expY = std(exp(Y(X==1)));
stdder0.expY = std(exp(Y(X==0)));

stdder1.Y = std(Y(X==1));
stdder0.Y = std(Y(X==0));

stdder1.W = std(W(X==1,:));
stdder0.W = std(W(X==0,:));

% Mean differences
meanDiff.Y = mean1.Y - mean0.Y;
meanDiff.expY = mean1.expY - mean0.expY;
meanDiff.W = mean1.W - mean0.W;

% Mean difference standard errors
meanDiffStdder.Y = sqrt( ((stdder1.Y)^2)/mean1.sampleSize ...
    + ((stdder0.Y)^2)/mean0.sampleSize );

meanDiffStdder.expY = sqrt( ((stdder1.expY)^2)/mean1.sampleSize ...
    + ((stdder0.expY)^2)/mean0.sampleSize );

meanDiffStdder.W = sqrt( ((stdder1.W).^2)/mean1.sampleSize ...
    + ((stdder0.W).^2)/mean0.sampleSize );

%% Print table
fid = fopen('../output/Empirical_meanComparisons.tex','w');

fprintf(fid, '\\begin{table}[H] \\centering');

fprintf(fid, '\\caption{Comparison of means \\label{empiricalMeanComparisons}} \n');

fprintf(fid, '\\setlength{\\linewidth}{.1cm} \n');
fprintf(fid, '\\newcommand{\\contents}{ \n');

fprintf(fid, '\\begin{tabular}{l r r r} \\hline\\hline \n');

fprintf(fid, 'Variable Name & Not Abducted & Abducted & Difference  \\\\ \\hline \n');

fprintf(fid, ' %s & %8.2f & %8.2f & %8.2f \\, [%4.2f] \\\\ \n ', ...
    expYlabel,mean1.expY,mean0.expY,meanDiff.expY,meanDiffStdder.expY); 

fprintf(fid, ' %s & %8.2f & %8.2f & %8.2f \\, [%4.2f] \\\\ \n ', ...
    Ylabel,mean1.Y,mean0.Y,meanDiff.Y,meanDiffStdder.Y); 

for j=1:size(W,2)
    Wvar = W(:,j);
    
    fprintf(fid, ' %s & %8.2f & %8.2f & %8.2f \\, [%4.2f] \\\\ \n ', ...
        Wlabels{j},mean1.W(:,j),mean0.W(:,j),...
            meanDiff.W(:,j),meanDiffStdder.W(:,j)); 
    
end

fprintf(fid, '\\hline \n');
fprintf(fid, 'Observations & %d & %d & \\\\ \n',mean1.sampleSize,mean0.sampleSize);

fprintf(fid, '\\hline\\hline \n');

fprintf(fid, '\\multicolumn{4}{p{0.95\\linewidth}}{\\footnotesize Sample size is %d. 1 USD is approximately 1800 Uganda shillings (Exchange rate at time of survey, 2005-2006; source: World Bank). Standard errors in brackets.} \\\\ \n',length(Y));
fprintf(fid, '\\end{tabular}\n');
fprintf(fid, '} \n'); 

fprintf(fid, '\\setbox0=\\hbox{\\contents} \n');
fprintf(fid, '\\setlength{\\linewidth}{\\wd0\\tabcolsep0.6em} \n');
fprintf(fid, '\\contents \n');

fprintf(fid, '\\end{table}\n');
fclose(fid);

end