% If no bandwidth vector is given then we use the empirical conditional 
% cdf. Otherwise, we use a logit smoothed cdf.
%
% Dvec = (number of grid points) by (number of bootstrap draws)
% That is, Dvec = (Ncgrid) by (Nbootstrap)
%
% k = (Ncgrid) by 1
function val = smoothedCDF(k,Dvec,hvec)
    
    B = size(Dvec,2);

    if nargin == 3
        
        kernelVec = nan(length(k),B);
             
        for i=1:length(k)
            Y = Dvec(i,:);
            y = k(i);
            h = hvec(i);

            kernelVec(i,:) = logitKernel(-(Y-y),h);
        end
        
        val = sum(prod(kernelVec,1))/B;
        
    elseif nargin == 2
        
        kmat = repmat(k,1,B);
        
        % Here we want to find the number of data points (corresponding to
        % columns of Dvec), for which ALL entries in that column are less
        % than the corresponding entry of k. Hence we want to pick out the
        % columns of (Dvec <= kmat) which have all ones.
        
        val = sum(sum((Dvec <= kmat),1) == length(k))/B;
    end        

end