% If (x=0 and hstruct.x0 = 0) or (x=1 and hstruct.x1=0) then we use the
% empirical conditional cdf. Otherwise, we use a logit smoothed cdf.
function val = smoothedConditionalCDF(y,x,Data,hstruct,WsuppPoint,DataBootstrap,epsilon)

    Y = Data.Y;
    X = Data.X;

    if nargin == 4
        
        % Allows a different bandwidth for each conditioning point
        if x == 0
            h = hstruct.x0;
        elseif x == 1
            h = hstruct.x1;
        end
        
        indicator = (X == x);
        
        if h > 0
            val = sum( logitKernel(-(Y-y),h).*indicator )/ sum(indicator);
        elseif h == 0
            val = sum( (Y <= y).*indicator )/ sum(indicator);
        end
        
    elseif nargin == 5

        W = Data.Wsingle;
        
        if x == 0
            h = hstruct.x0cond(WsuppPoint);
        elseif x == 1
            h = hstruct.x1cond(WsuppPoint);
        end
        
        indicator = (X == x).*(W == WsuppPoint);
        
        if h > 0
            val = sum( logitKernel(-(Y-y),h).*indicator )/ sum(indicator);
        elseif h == 0
            val = sum( (Y <= y).*indicator )/ sum(indicator);
        end
        
    elseif nargin == 7
        
        if Data.covariatesOn == 0

            if x == 0
                h = hstruct.x0;
            elseif x == 1
                h = hstruct.x1;
            end        

            if h > 0

                Fhat = sum( logitKernel(-(Y-y),h).*(X == x) )/ sum( (X==x) );

                Ystar = DataBootstrap.Y;
                Xstar = DataBootstrap.X;
                n = length(Ystar);

                Fhatstar = ...
                  sum(logitKernel(-(Ystar-y),h).*(Xstar == x))/sum((Xstar==x));

                val = Fhat + epsilon*sqrt(n)*(Fhatstar - Fhat);

            elseif h == 0

                [val,~] = computePerturbedCDF(y,x,[],Data,DataBootstrap,epsilon);

            end

        elseif Data.covariatesOn == 1
            
            W = Data.Wsingle;

            if x == 0
                h = hstruct.x0cond(WsuppPoint);
            elseif x == 1
                h = hstruct.x1cond(WsuppPoint);
            end

            if h > 0
                
                indicator = (X == x).*(W == WsuppPoint);

                Fhat = sum( logitKernel(-(Y-y),h).*indicator )/ sum(indicator);

                Ystar = DataBootstrap.Y;
                Xstar = DataBootstrap.X;
                Wstar = DataBootstrap.Wsingle;
                n = length(Ystar);
                
                indicatorStar = (Xstar == x).*(Wstar == WsuppPoint);

                Fhatstar = ...
                  sum(logitKernel(-(Ystar-y),h).*indicatorStar)/sum(indicatorStar);

                val = Fhat + epsilon*sqrt(n)*(Fhatstar - Fhat);

            elseif h == 0

                [val,~] = computePerturbedCDF(y,x,WsuppPoint,Data,DataBootstrap,epsilon);

            end
            
        end
       
    else
        error('Arguments not correct in smoothedConditionalCDF.m');
        
    end
    
end