% WARNING: ygrid here must match the ygrid defined in Qupper.m and Qlower.m
%
function DataBootstrap = storeFbounds(params,cgrid,h,Data,DataBootstrap,epsilon)

    XsuppSize = 2;
    Nygrid = 100;
    ygrid = linspace(min(Data.Y),max(Data.Y),Nygrid);

    tauvalsLower = nan(length(ygrid),XsuppSize,length(cgrid));
    tauvalsUpper = nan(length(ygrid),XsuppSize,length(cgrid));
    
    tauvalsLower_CondW = nan(length(ygrid),XsuppSize,length(cgrid),Data.WsingleSuppSize);
    tauvalsUpper_CondW = nan(length(ygrid),XsuppSize,length(cgrid),Data.WsingleSuppSize);

    for j=1:XsuppSize
        xEval = j-1;
        
        for k=1:length(cgrid)
            c = cgrid(k);
        
            for i=1:length(ygrid)
                y = ygrid(i);

                % Compute UNCONDITIONAL on W cdf bounds
                tauvalsLower(i,j,k) = ...
                    Flower(params,y,xEval,c,Data,h,DataBootstrap,epsilon);  
                
                tauvalsUpper(i,j,k) = ...
                    Fupper(params,y,xEval,c,Data,h,DataBootstrap,epsilon);      
                
                % Compute CONDITIONAL on W cdf bounds
                for l=1:Data.WsingleSuppSize
                    wEval = l;

                    % The flow control here is complicated. Here we only
                    % want to get the cdf conditional on a single x and
                    % single w. But Flower was not really well written for
                    % this. So the last argument of Flower overrides the
                    % default Flower code. The key issue is that we have
                    % already computed the perturbed cdfs, so we do NOT
                    % want to compute them again, and so we need Flower to
                    % just grab those pre-computed cdfs properly. This
                    % complicates the code.
                    tauvalsLower_CondW(i,j,k,l) = ...
                        Flower(params,y,xEval,c,Data,h,DataBootstrap,epsilon,wEval);  

                    tauvalsUpper_CondW(i,j,k,l) = ...
                        Fupper(params,y,xEval,c,Data,h,DataBootstrap,epsilon,wEval); 
                    
                end

            end
        end
    end
    
    DataBootstrap.Flower = tauvalsLower;
    DataBootstrap.Fupper = tauvalsUpper;   
    
    DataBootstrap.FlowerCondW = tauvalsLower_CondW;
    DataBootstrap.FupperCondW = tauvalsUpper_CondW;   

end