% In this function, tau and x can be vectors.
% This is used, for example, in generating data for the Monte Carlo study
%
% If tau < 0 then this function returns the 0.00-quantile.
% Likewise, if tau > 1 then this function returns the 1.00-quantile.
function quantile = trueQuantileFunction_MC(params,tau,x,WsuppPoint)

    distChoice = 1;

    if distChoice == 1
        a = -4;
        b = 4;
        
        normaldist = makedist('normal');
        truncatedNormalDist = truncate(normaldist,a,b);
        
        dist = truncatedNormalDist;
        
    end

    % Extend definition of the function to entire support in a logical way
    if tau < 0
        tau = 0;
    elseif tau > 1
        tau = 1;
    end
    
    if nargin == 3

        meanTerm = params.PI*x;
        stddevTerm = params.gamma*x + 1;
                
    elseif nargin == 4
        
        meanTerm = params.PI*x + params.PIW*WsuppPoint;
        stddevTerm = params.gamma*x + params.gammaW*WsuppPoint + params.sigma;
        
    end

    quantile = meanTerm + stddevTerm.*icdf(dist,tau);
end
